"""Hive Action Module."""
import asyncio
from typing import Optional
from aiohttp import ClientSession

from .hive_session import Session
from .hive_data import Data
from .custom_logging import Logger
from .device_attributes import Attributes
from .hive_async_api import Hive_Async


class Action:
    """Hive Action Code."""

    def __init__(self, websession: Optional[ClientSession] = None):
        """Initialise."""
        self.hive = Hive_Async(websession)
        self.session = Session(websession)
        self.log = Logger()
        self.attr = Attributes()
        self.type = "Action"

    async def get_action(self, device):
        """Get smart plug current power usage."""
        await self.log.log(device["hive_id"], self.type, "Getting action data.")
        dev_data = {}

        if device["hive_id"] in Data.actions:
            data = Data.actions[device["hive_id"]]
            dev_data = {"hive_id": device["hive_id"],
                        "hive_name": device["hive_name"],
                        "hive_type": device["hive_type"],
                        "ha_name": device["ha_name"],
                        "ha_type": device["ha_type"],
                        "state": await self.get_state(device),
                        "power_usage": None,
                        "device_data": {},
                        "custom": device.get("custom", None)
                        }

        await self.log.log(device["hive_id"], self.type,
                           "action update {0}", info=dev_data)
        return dev_data

    async def get_state(self, device):
        """Get action state."""
        await self.log.log(device["hive_id"], "Extra", "Getting state")
        state = None
        final = None

        if device["hive_id"] in Data.actions:
            data = Data.actions[device["hive_id"]]
            final = data["enabled"]
            Data.NODES[device["hive_id"]]["State"] = final
            await self.log.log(device["hive_id"], "Extra", "Status is {0}", info=final)
            if device["hive_id"] in Data. s_error_list:
                Data. s_error_list.popitem(device["hive_id"])
        else:
            await self.log.error_check(device["hive_id"], "ERROR", "Failed")

        return final if final is None else Data.NODES[device["hive_id"]].get("State")

    async def turn_on(self, device):
        """Set action turn on."""
        import json
        await self.log.log(device["hive_id"], "Extra", "Enabling action")
        final = False

        if device["hive_id"] in Data.actions:
            await self.session.hive_refresh_tokens()
            data = Data.actions[device["hive_id"]]
            data.update({"enabled": True})
            send = json.dumps(data)
            resp = await self.hive.set_action(device["hive_id"], send)
            if resp["original"] == 200:
                final = True
                await self.session.get_devices(device["hive_id"])
                await self.log.log(device["hive_id"], "API", "Enabled action - " + device["hive_name"])
            else:
                await self.log.error_check(
                    device["hive_id"], "ERROR", "Failed_API", resp=resp["original"])

        return final

    async def turn_off(self, device):
        """Set action to turn off."""
        import json
        await self.log.log(device["hive_id"], "Extra", "Disabling action")
        final = False

        if device["hive_id"] in Data.actions:
            await self.session.hive_refresh_tokens()
            data = Data.actions[device["hive_id"]]
            data.update({"enabled": False})
            send = json.dumps(data)
            resp = await self.hive.set_action(device["hive_id"], send)
            if resp["original"] == 200:
                final = True
                await self.session.get_devices(device["hive_id"])
                await self.log.log(device["hive_id"], "API", "Disabled action - " + device["hive_name"])
            else:
                await self.log.error_check(
                    device["hive_id"], "ERROR", "Failed_API", resp=resp["original"])

        return final



"""Hive Weather Module."""
import asyncio
from .custom_logging import Logger
from .hive_data import Data


class Weather:
    """Hive Weather Code."""

    def __init__(self):
        self.log = Logger()

    async def temperature(self, device):
        """Get Hive Weather temperature."""
        await self.log.log("Weather", "Temp", "Getting outside temp.")
        return Data.w_temperature_value




from .hive_data import Data


class Hive_Helper:

    def __init__():
        """intialise the helper class."""

    def get_device_name(n_id):
        """Resolve a id into a name"""
        try:
            product_name = Data.products[n_id]["state"]["name"]
        except KeyError:
            product_name = False

        try:
            device_name = Data.devices[n_id]["state"]["name"]
        except KeyError:
            device_name = False

        if product_name:
            return product_name
        elif device_name:
            return device_name
        elif n_id == "No_ID":
            return "Hive"
        else:
            return 'UNKNOWN'



"""Hive API Module."""
import asyncio
from .hive_exceptions import FileInUse

from aiohttp import ClientSession, ClientResponse
from typing import Optional
from .custom_logging import Logger
from .hive_data import Data


class Hive_Async:
    """Hive API Code."""

    def __init__(self, websession: Optional[ClientSession] = None):
        """Hive API initialisation."""
        self.log = Logger()
        self.urls = {
            "login": "https://beekeeper.hivehome.com/1.0/cognito/login",
            "refresh": "https://beekeeper.hivehome.com/1.0/cognito/refresh-token",
            "long_lived": "https://api.prod.bgchprod.info/omnia/accessTokens",
            "base": "https://beekeeper-uk.hivehome.com/1.0",
            "weather": "https://weather.prod.bgchprod.info/weather",
            "holiday_mode": "/holiday-mode",
            "all": "/nodes/all?products=true&devices=true&actions=true",
            "devices": "/devices",
            "products": "/products",
            "actions": "/actions",
            "nodes": "/nodes/{0}/{1}",
        }
        self.headers = {
            "content-type": "application/json",
            "Accept": "*/*",
        }
        self.timeout = 10
        self.json_return = {
            "original": "No response to Hive API request",
            "parsed": "No response to Hive API request",
        }
        self.websession = ClientSession = websession
        self.api_lock = asyncio.Lock()

    async def request(self, method: str, url: str, **kwargs) -> ClientResponse:
        """Make a request."""
        if self.api_lock.locked():
            return True

        await self.api_lock.acquire()
        data = kwargs.get('data', None)
        await self.log.log(
            'API', 'API', ("Request is - {0}:{1}".format(method, url)))
        async with self.websession.request(method, url, headers=self.headers, data=data) as resp:
            if method != "delete":
                await resp.json(content_type=None)
                self.json_return.update({"original": resp.status})
                self.json_return.update({"parsed": await resp.json(content_type=None)})
                await self.log.log('API', 'API', "Response is - " + str(resp.status))

        self.api_lock.release()

    async def login(self, username, password):
        """Login to the Hive API."""
        url = self.urls["login"]
        j = '{{"username": "{0}", "password": "{1}"}}'.format(
            username, password)
        try:
            await self.request('post', url, data=j)
        except (ConnectionError, IOError, RuntimeError, ZeroDivisionError):
            await self.error()

        return self.json_return

    async def refresh_tokens(self):
        """Refresh tokens"""
        url = self.urls["refresh"]
        jsc = (
            "{"
            + ",".join(
                ('"' + str(i) + '": ' '"' + str(t) +
                 '" ' for i, t in Data.s_tokens.items())
            )
            + "}"
        )
        try:
            await self.request('post', url,  data=jsc)
        except (ConnectionError, IOError, RuntimeError, ZeroDivisionError):
            await self.error()

        return self.json_return

    async def check_token(self, token):
        """Get a long lived token"""
        self.headers.update({"authorization": "Bearer " + token})
        self.headers.update({"X-Omnia-Client": "swagger"})
        self.headers.update({"X-MQTT-Client-ID": "swagger_MQTT_client"})
        self.headers.update(
            {"Accept": "application/vnd.alertme.zoo-6.6.0+json"})
        url = self.urls["long_lived"]
        try:
            await self.request('get', url)
        except (ConnectionError, IOError, RuntimeError, ZeroDivisionError):
            await self.error()

        return self.json_return

    async def create_token(self, token):
        """Get a long lived token"""
        self.headers.update({"authorization": "Bearer " + token})
        self.headers.update({"X-Omnia-Client": "swagger"})
        self.headers.update({"X-MQTT-Client-ID": "swagger_MQTT_client"})
        self.headers.update(
            {"Accept": "application/vnd.alertme.zoo-6.6.0+json"})
        url = self.urls["long_lived"]
        j = '{ "accessTokens": [ { "description": "HA access token" } ] }'
        try:
            await self.request('post', url,  data=j)
        except (ConnectionError, IOError, RuntimeError, ZeroDivisionError):
            await self.error()

        return self.json_return

    async def remove_token(self, token, token_id):
        """Get a long lived token"""
        self.headers.update({"X-Omnia-Access-Token": token})
        self.headers.update({"X-Omnia-Client": "swagger"})
        self.headers.update({"X-MQTT-Client-ID": "swagger_MQTT_client"})
        self.headers.update(
            {"Accept": "application/vnd.alertme.zoo-6.6.0+json"})
        url = self.urls["long_lived"] + "/" + token_id
        try:
            await self.request('delete', url)
        except (ConnectionError, IOError, RuntimeError, ZeroDivisionError):
            await self.error()

        return self.json_return

    async def get_all(self):
        """Build and query all endpoint."""
        url = self.urls["base"] + self.urls["all"]
        try:
            await self.request('get', url)
        except (IOError, RuntimeError, ZeroDivisionError):
            await self.error()

        return self.json_return

    async def get_devices(self):
        """Call the get devices endpoint."""
        url = self.urls["base"] + self.urls["devices"]
        try:
            await self.request('get', url)
        except (IOError, RuntimeError, ZeroDivisionError):
            await self.error()

        return self.json_return

    async def get_products(self, token):
        """Call the get products endpoint."""
        url = self.urls["base"] + self.urls["products"]
        try:
            await self.request('get', url)
        except (IOError, RuntimeError, ZeroDivisionError):
            await self.error()

        return self.json_return

    async def get_actions(self):
        """Call the get actions endpoint."""
        url = self.urls["base"] + self.urls["actions"]
        try:
            await self.request('get', url)
        except (IOError, RuntimeError, ZeroDivisionError):
            await self.error()

        return self.json_return

    async def motion_sensor(self, token, sensor, fromepoch, toepoch):
        """Call a way to get motion sensor info."""
        url = (
            self.urls["base"]
            + self.urls["products"]
            + "/"
            + sensor["type"]
            + "/"
            + sensor["id"]
            + "/events?from="
            + str(fromepoch)
            + "&to="
            + str(toepoch)
        )
        try:
            await self.request('get', url)
        except (IOError, RuntimeError, ZeroDivisionError):
            await self.error()

        return self.json_return

    async def get_weather(self, weather_url):
        """Call endpoint to get local weather from Hive API."""
        t_url = self.urls["weather"] + weather_url
        url = t_url.replace(" ", "%20")
        try:
            await self.request('get', url)
        except (IOError, RuntimeError, ZeroDivisionError, ConnectionError):
            await self.error()

        return self.json_return

    async def set_state(self, n_type, n_id, **kwargs):
        """Set the state of a Device."""
        jsc = (
            "{"
            + ",".join(
                ('"' + str(i) + '": ' '"' + str(t) +
                 '" ' for i, t in kwargs.items())
            )
            + "}"
        )

        url = self.urls["base"] + self.urls["nodes"].format(n_type, n_id)
        try:
            await self.is_file_being_used()
            await self.request('post', url,  data=jsc)
        except (FileInUse, IOError, RuntimeError, ConnectionError)as e:
            if e.__class__.__name__ == "FileInUse":
                return {"original": "file"}
            else:
                await self.error()

        return self.json_return

    async def set_action(self, n_id, data):
        """Set the state of a Action."""
        jsc = data
        url = self.urls["base"] + self.urls["actions"] + "/" + n_id
        try:
            await self.is_file_being_used()
            await self.request('put', url,  data=jsc)
        except (FileInUse, IOError, RuntimeError, ConnectionError) as e:
            if e.__class__.__name__ == "FileInUse":
                return {"original": "file"}
            else:
                await self.error()

        return self.json_return

    async def error(self):
        """An error has occured iteracting wth the Hive API."""
        self.json_return.update({"original": "Error making API call"})
        self.json_return.update({"parsed": "Error making API call"})
        await self.log.log("API_ERROR", "ERROR", "Error attempting API call")

    async def is_file_being_used(self):
        """Check if running in file mode."""
        if Data.s_file:
            raise FileInUse()



"""Hive Sensor Module."""
import asyncio
from .hive_data import Data
from .custom_logging import Logger
from .device_attributes import Attributes
from .weather import Weather
from .hub import Hub
from .heating import Heating
from .hotwater import Hotwater


class Sensor:
    """Hive Sensor Code."""

    def __init__(self):
        """Initialise."""
        self.log = Logger()
        self.attributes = Attributes()
        self.weather = Weather()
        self.hub = Hub()
        self.heating = Heating()
        self.hotwater = Hotwater()
        self.type = "Sensor"

    async def get_sensor(self, device):
        await self.log.log(device["hive_id"], self.type, "Getting sensor data.")
        online = await self.attributes.online_offline(device["hive_id"])
        error = await self.log.error_check(device["hive_id"], self.type, online)

        dev_data = {}
        dev_data = {"hive_id": device["hive_id"],
                    "hive_name": device["hive_name"],
                    "hive_type": device["hive_type"],
                    "ha_name": device["ha_name"],
                    "ha_type": device["ha_type"],
                    "device_id": device.get("device_id", None),
                    "device_name": device.get("device_name", None),
                    "hub_id": device.get("hub_id", None),
                    "device_data": {},
                    "custom": device.get("custom", None)
                    }

        data = {}
        if dev_data["device_id"] in Data.devices:
            data = Data.devices.get(device["device_id"], {})
        elif dev_data["hive_id"] in Data.products:
            data = Data.products.get(device["hive_id"], {})

        if dev_data["hive_type"] in Data.sensor_commands or \
                dev_data.get("custom", None) in Data.sensor_commands:
            code = Data.sensor_commands.get(
                dev_data["hive_type"], Data.sensor_commands.get(dev_data["custom"]))
            dev_data.update({"state": await eval(code),
                             "device_data": data.get("props", None),
                             "parent_device": data.get("parent", None)})
        elif device["hive_type"] in Data.HIVE_TYPES["Sensor"]:
            data = Data.devices.get(device["hive_id"], {})
            dev_data.update({"state": await self.get_state(device),
                             "device_data": data.get("props", None),
                             "parent_device": data.get("parent", None),
                             "attributes": await self.attributes.state_attributes(device["hive_id"],
                                                                                  device["hive_type"])})
        if not error:
            await self.log.log(device["hive_id"], self.type,
                               "Device update {0}", info=dev_data)

        return dev_data

    async def get_state(self, device):
        """Get sensor state."""
        await self.log.log(device["hive_id"], "Extra", "Getting state")
        online = await self.attributes.online_offline(device["hive_id"])
        state = None
        state = None
        final = None

        if device["hive_id"] in Data.products:
            if online:
                data = Data.products[device["hive_id"]]
                if data["type"] == "contactsensor":
                    state = data["props"]["status"]
                elif data["type"] == "motionsensor":
                    state = data["props"]["motion"]["status"]
                final = Data.HIVETOHA[self.type].get(state, state)
                await self.log.log(device["hive_id"], "Extra", "Status is {0}", info=final)
                if device["hive_id"] in Data. s_error_list:
                    Data. s_error_list.popitem(device["hive_id"])
            await self.log.error_check(device["hive_id"], "Extra", online)
            final = Data.HIVETOHA[self.type].get(state, state)
            Data.NODES[device["hive_id"]]["State"] = final
        else:
            await self.log.error_check(device["hive_id"], "ERROR", "Failed")

        return final if final is None else Data.NODES[device["hive_id"]]["State"]

    async def online(self, device):
        """Get the online status of the Hive hub."""
        await self.log.log(device["hive_id"], "Extra", "Getting status")
        state = None
        final = None

        if device["hive_id"] in Data.devices:
            data = Data.devices[device["hive_id"]]
            state = data["props"]["online"]
            final = Data.HIVETOHA[self.type].get(state, state)
            await self.log.log(device["hive_id"], "Extra", "Status is {0}", info=final)
            await self.log.error_check(device["hive_id"], "Extra", state)
            final = Data.HIVETOHA[self.type].get(state, state)
            Data.NODES[device["hive_id"]]["State"] = final
            if device["hive_id"] in Data. s_error_list:
                Data. s_error_list.popitem(device["hive_id"])
        else:
            await self.log.error_check(device["hive_id"], "ERROR", "Failed")

        return final if final is None else Data.NODES[device["hive_id"]]["State"]



"""__init__.py"""
from .const import *
from .hive_session import Session
from .hiveauth import HiveAuth
from .action import Action
from .client import Client
from .custom_logging import Logger
from .hive_api import Hive
from .device_attributes import Attributes
from .heating import Heating
from .hive_async_api import Hive_Async
from .hive_data import Data
from .hotwater import Hotwater
from .hub import Hub
from .light import Light
from .plug import Plug
from .sensor import Sensor
from .weather import Weather


from cryptography.fernet import Fernet
import os
import json


def get_message(message, type):
    """
    Gets a message
    """
    key = open(os.path.dirname(os.path.realpath(__file__)) + "/.info.key", "rb").read()
    f = Fernet(key)
    result = f.decrypt(message).decode()

    if type == "JSON":
        result = json.loads(result)
    return result



"""Hive API Module."""
import requests
from .custom_logging import Logger


class Hive:
    """Hive API Code."""

    def __init__(self):
        """Hive API initialisation."""
        self.log = Logger()
        self.urls = {
            "login": "https://beekeeper.hivehome.com/1.0/cognito/login",
            "long_lived": "https://api.prod.bgchprod.info/omnia/accessTokens",
            "base": "https://beekeeper-uk.hivehome.com/1.0",
            "weather": "https://weather.prod.bgchprod.info/weather",
            "holiday_mode": "/holiday-mode",
            "all": "/nodes/all?products=true&devices=true&actions=true",
            "devices": "/devices",
            "products": "/products",
            "actions": "/actions",
            "nodes": "/nodes/{0}/{1}",
        }
        self.headers = {
            "content-type": "application/json",
            "Accept": "*/*",
            "authorization": "None",
        }
        self.timeout = 10
        self.json_return = {
            "original": "No response to Hive API request",
            "parsed": "No response to Hive API request",
        }

    def login(self, username, password):
        """Login to the Hive API."""
        try:
            j = '{{"username": "{0}", "password": "{1}"}}'.format(
                username, password)
            response = requests.post(
                url=self.urls["login"],
                headers=self.headers,
                data=j,
                timeout=self.timeout,
            )
            self.json_return.update({"original": response.status_code})
            self.json_return.update({"parsed": response.json()})
        except (ConnectionError, IOError, RuntimeError, ZeroDivisionError):
            self.error()

        return self.json_return

    def check_token(self, token):
        """Get a long lived token"""
        self.headers.update({"authorization": "Bearer " + token})
        self.headers.update({"X-Omnia-Client": "swagger"})
        self.headers.update({"X-MQTT-Client-ID": "swagger_MQTT_client"})
        self.headers.update(
            {"Accept": "application/vnd.alertme.zoo-6.6.0+json"})
        try:
            response = requests.get(
                url=self.urls["long_lived"],
                headers=self.headers,
                data=None,
                timeout=self.timeout,
            )
            self.json_return.update({"original": response.status_code})
            self.json_return.update({"parsed": response.json()})
        except (ConnectionError, IOError, RuntimeError, ZeroDivisionError):
            self.error()

        return self.json_return

    def create_token(self, token):
        """Get a long lived token"""
        self.headers.update({"authorization": "Bearer " + token})
        self.headers.update({"X-Omnia-Client": "swagger"})
        self.headers.update({"X-MQTT-Client-ID": "swagger_MQTT_client"})
        self.headers.update(
            {"Accept": "application/vnd.alertme.zoo-6.6.0+json"})
        try:
            j = '{ "accessTokens": [ { "description": "HA access token" } ] }'
            response = requests.post(
                url=self.urls["long_lived"],
                headers=self.headers,
                data=j,
                timeout=self.timeout,
            )
            self.json_return.update({"original": response.status_code})
            self.json_return.update({"parsed": response.json()})
        except (ConnectionError, IOError, RuntimeError, ZeroDivisionError):
            self.error()

        return self.json_return

    def remove_token(self, token, token_id):
        """Get a long lived token"""
        self.headers.update({"X-Omnia-Access-Token": token})
        self.headers.update({"X-Omnia-Client": "swagger"})
        self.headers.update({"X-MQTT-Client-ID": "swagger_MQTT_client"})
        self.headers.update(
            {"Accept": "application/vnd.alertme.zoo-6.6.0+json"})
        try:
            response = requests.delete(
                url=self.urls["long_lived"] + "/" + token_id,
                headers=self.headers,
                data=None,
                timeout=self.timeout,
            )
            self.json_return.update({"original": response.status_code})
            self.json_return.update({"parsed": response.json()})
        except (ConnectionError, IOError, RuntimeError, ZeroDivisionError):
            self.error()

        return self.json_return

    def get_all(self, session_id):
        """Build and query all endpoint."""
        self.headers.update({"authorization": session_id})
        url = self.urls["base"] + self.urls["all"]
        try:
            jsc = None
            response = requests.get(
                url=url, headers=self.headers, data=jsc, timeout=self.timeout
            )
            self.json_return.update({"original": response.status_code})
            self.json_return.update({"parsed": response.json()})
        except (IOError, RuntimeError, ZeroDivisionError):
            self.error()

        return self.json_return

    def get_devices(self, session_id):
        """Call the get devices endpoint."""
        self.headers.update({"authorization": session_id})
        url = self.urls["base"] + self.urls["devices"]
        try:
            jsc = None
            response = requests.get(
                url=url, headers=self.headers, data=jsc, timeout=self.timeout
            )
            self.json_return.update({"original": response.status_code})
            self.json_return.update({"parsed": response.json()})
        except (IOError, RuntimeError, ZeroDivisionError):
            self.error()

        return self.json_return

    def get_products(self, session_id):
        """Call the get products endpoint."""
        self.headers.update({"authorization": session_id})
        url = self.urls["base"] + self.urls["products"]
        try:
            jsc = None
            response = requests.get(
                url=url, headers=self.headers, data=jsc, timeout=self.timeout
            )
            self.json_return.update({"original": response.status_code})
            self.json_return.update({"parsed": response.json()})
        except (IOError, RuntimeError, ZeroDivisionError):
            self.error()

        return self.json_return

    def get_actions(self, session_id):
        """Call the get actions endpoint."""
        self.headers.update({"authorization": session_id})
        url = self.urls["base"] + self.urls["actions"]
        try:
            jsc = None
            response = requests.get(
                url=url, headers=self.headers, data=jsc, timeout=self.timeout
            )
            self.json_return.update({"original": response.status_code})
            self.json_return.update({"parsed": response.json()})
        except (IOError, RuntimeError, ZeroDivisionError):
            self.error()

        return self.json_return

    def motion_sensor(self, session_id, sensor, fromepoch, toepoch):
        """Call a way to get motion sensor info."""
        self.headers.update({"authorization": session_id})
        url = (
            self.urls["base"]
            + self.urls["products"]
            + "/"
            + sensor["type"]
            + "/"
            + sensor["id"]
            + "/events?from="
            + str(fromepoch)
            + "&to="
            + str(toepoch)
        )
        try:
            jsc = None
            response = requests.get(
                url=url, headers=self.headers, data=jsc, timeout=self.timeout
            )
            self.json_return.update({"original": response.status_code})
            self.json_return.update({"parsed": response.json()})
        except (IOError, RuntimeError, ZeroDivisionError):
            self.error()

        return self.json_return

    def get_weather(self, weather_url):
        """Call endpoint to get local weather from Hive API."""
        t_url = self.urls["weather"] + weather_url
        url = t_url.replace(" ", "%20")
        try:
            jsc = None
            response = requests.get(
                url=url, headers=self.headers, data=jsc, timeout=self.timeout
            )
            self.json_return.update({"original": response.status_code})
            self.json_return.update({"parsed": response.json()})
        except (IOError, RuntimeError, ZeroDivisionError, ConnectionError):
            self.error()

        return self.json_return

    def set_state(self, session_id, n_type, n_id, **kwargs):
        """Set the state of a Device."""
        self.headers.update({"authorization": session_id})
        jsc = (
            "{"
            + ",".join(
                ('"' + str(i) + '": ' '"' + str(t) +
                 '" ' for i, t in kwargs.items())
            )
            + "}"
        )

        url = self.urls["base"] + self.urls["nodes"].format(n_type, n_id)

        self.log.log(
            n_id,
            "api_core",
            "Headers >\n{0}\nURL >\n{1}\n"
            + "Payload\n{2}\n".format(self.headers, url, jsc),
        )
        try:
            response = requests.post(
                url=url, headers=self.headers, data=jsc, timeout=self.timeout
            )
            self.json_return.update({"original": response.status_code})
            self.json_return.update({"parsed": response.json()})
        except (IOError, RuntimeError, ZeroDivisionError, ConnectionError):
            self.error()

        return self.json_return

    def set_action(self, session_id, n_id, data):
        """Set the state of a Action."""
        self.headers.update({"authorization": session_id})
        jsc = data
        url = self.urls["base"] + self.urls["actions"] + "/" + n_id
        self.log.log(
            n_id,
            "api_core",
            "Headers >\n{0}\nURL >\n{1}\n"
            + "Payload\n{2}\n".format(self.headers, url, jsc),
        )
        try:
            response = requests.put(
                url=url, headers=self.headers, data=jsc, timeout=self.timeout
            )
            self.json_return.update({"original": response.status_code})
            self.json_return.update({"parsed": response.json()})
        except (IOError, RuntimeError, ZeroDivisionError, ConnectionError):
            self.error()

        return self.json_return

    def error(self):
        """An error has occured iteracting wth the Hive API."""
        self.json_return.update({"original": "Error making API call"})
        self.json_return.update({"parsed": "Error making API call"})
        self.log.log("API_ERROR", "ERROR", "Error attempting API call")



"""Custom Logging Module."""
import logging

from datetime import datetime, timedelta
from .hive_data import Data
from.helper import Hive_Helper

_LOGGER = logging.getLogger(__name__)


class Logger:
    """Custom Logging Code."""

    def __init__(self):
        """Logger Initialization"""
        self.last_logged = None

    @staticmethod
    async def check_debuging(enable_debug: list):
        """Check Logging Active"""

        if len(enable_debug) > 0:
            Data.d_enabled = True
        else:
            Data.d_enabled = False

        Data.d_list = enable_debug
        return Data.d_enabled

    @staticmethod
    async def log(n_id, l_type, new_message, **kwargs):
        """Output new log entry if logging is turned on."""
        name = Hive_Helper.get_device_name(n_id)
        data = kwargs.get("info", None)
        if "_" in l_type:
            nxt = l_type.split("_")
            for i in nxt:
                if i in Data.d_list or "All" in Data.d_list and l_type != "Extra":
                    break

        if Data.d_enabled and new_message is not None and any(elem in Data.d_list for elem in [l_type, 'All']):
            if l_type != "ERROR":
                _LOGGER.debug(new_message.format(data))

            try:
                l_file = open(Data.d_o_file, "a")
                l_file.write(
                    datetime.now().strftime("%d-%b-%Y %H:%M:%S")
                    + " - "
                    + l_type
                    + " - "
                    + name
                    + " : "
                    + new_message.format(data)
                    + "\n"
                )
                l_file.close()
            except FileNotFoundError:
                pass
        else:
            pass

    async def error_check(self, n_id, n_type, error_type, **kwargs):
        """Error has occurred."""
        message = None
        new_data = None
        result = False
        name = Hive_Helper.get_device_name(n_id)

        if error_type == False:
            message = "Device offline could not update entity - " + name
            result = True
            if Data.s_file and n_id not in Data. s_error_list:
                _LOGGER.warning(message)
                Data. s_error_list.update({n_id: datetime.now()})
        elif error_type == "Failed":
            message = "ERROR - No data found for device - " + name
            result = True
            if Data.s_file and n_id not in Data. s_error_list:
                _LOGGER.error(message)
                Data. s_error_list.update({n_id: datetime.now()})
        elif error_type == "Failed_API":
            new_data = str(kwargs.get("resp"))
            message = "ERROR - Received {0} response from API."
            result = True
            if not Data.s_file:
                _LOGGER.error(message.format(new_data))

        await self.log(n_id, n_type, message, info=new_data)
        return result



"""Hive Light Module."""
import colorsys
import asyncio
from typing import Optional
from aiohttp import ClientSession

from .hive_session import Session
from .hive_data import Data
from .custom_logging import Logger
from .device_attributes import Attributes
from .hive_async_api import Hive_Async


class Light:
    """Hive Light Code."""

    def __init__(self, websession: Optional[ClientSession] = None):
        """Initialise."""
        self.hive = Hive_Async(websession)
        self.session = Session(websession)
        self.log = Logger()
        self.attr = Attributes()
        self.type = "Light"

    async def get_light(self, device):
        """Get light data."""
        await self.log.log(device["hive_id"], self.type, "Getting light data.")
        online = await self.attr.online_offline(device["hive_id"])
        error = await self.log.error_check(device["hive_id"], self.type, online)

        dev_data = {}
        if device["hive_id"] in Data.devices:
            data = Data.devices[device["hive_id"]]
            dev_data = {"hive_id": device["hive_id"],
                        "hive_name": device["hive_name"],
                        "hive_type": device["hive_type"],
                        "ha_name": device["ha_name"],
                        "ha_type": device["ha_type"],
                        "device_id": device["device_id"],
                        "device_name": device["device_name"],
                        "state": await self.get_state(device),
                        "brightness": await self.get_brightness(device),
                        "device_data": data.get("props", None),
                        "parent_device": data.get("parent", None),
                        "custom": device.get("custom", None),
                        "attributes": await self.attr.state_attributes(device["hive_id"],
                                                                       device["hive_type"])
                        }

            if device["hive_type"] in ("tuneablelight", "colourtuneablelight"):
                dev_data.update({"min_mireds": await self.get_min_color_temp(device),
                                 "max_mireds": await self.get_max_color_temp(device),
                                 "color_temp": await self.get_color_temp(device)})
            if device["hive_type"] == "colourtuneablelight":
                dev_data.update({"hs_color": await self.get_color(device)})

        if not error:
            await self.log.log(device["hive_id"], self.type,
                               "Device update {0}", info=dev_data)

        return dev_data

    async def get_state(self, device):
        """Get light current state."""
        await self.log.log(device["hive_id"], "Extra", "Getting state")
        online = await self.attr.online_offline(device["hive_id"])
        state = None
        state = None
        final = None

        if device["hive_id"] in Data.products:
            if online:
                data = Data.products[device["hive_id"]]
                state = data["state"]["status"]
                await self.log.log(device["hive_id"], "Extra", "Status is {0}", info=state)
                if device["hive_id"] in Data. s_error_list:
                    Data. s_error_list.popitem(device["hive_id"])
            await self.log.error_check(device["hive_id"], "Extra", online)
            final = Data.HIVETOHA[self.type].get(state, state)
            Data.NODES[device["hive_id"]]["State"] = final
        else:
            await self.log.error_check(device["hive_id"], "ERROR", "Failed")

        return final if final is None else Data.NODES[device["hive_id"]]["State"]

    async def get_brightness(self, device):
        """Get light current brightness."""
        await self.log.log(device["hive_id"], "Extra", "Getting brightness")
        online = await self.attr.online_offline(device["hive_id"])
        state = None
        final = None

        if device["hive_id"] in Data.products:
            if online:
                data = Data.products[device["hive_id"]]
                state = data["state"]["brightness"]
                final = (state / 100) * 255
                Data.NODES[device["hive_id"]]["Brightness"] = final
                await self.log.log(device["hive_id"], "Extra", "Brightness is {0}", info=final)
                if device["hive_id"] in Data. s_error_list:
                    Data. s_error_list.popitem(device["hive_id"])
            await self.log.error_check(device["hive_id"], "Extra", online)
        else:
            await self.log.error_check(device["hive_id"], "ERROR", "Failed")

        return final if final is None else Data.NODES[device["hive_id"]]["Brightness"]

    async def get_min_color_temp(self, device):
        """Get light minimum color temperature."""
        await self.log.log(device["hive_id"], "Extra", "Getting min colour temperature")
        online = await self.attr.online_offline(device["hive_id"])
        state = None
        final = None

        if device["hive_id"] in Data.products:
            if online:
                data = Data.products[device["hive_id"]]
                state = data["props"]["colourTemperature"]["max"]
                final = round((1 / state) * 1000000)
                Data.NODES[device["hive_id"]]["Min_CT"] = final
                await self.log.log(device["hive_id"], "Extra",
                                   "Min colour temp is {0}", info=final)
                if device["hive_id"] in Data. s_error_list:
                    Data. s_error_list.popitem(device["hive_id"])
            await self.log.error_check(device["hive_id"], "Extra", online)
        else:
            await self.log.error_check(device["hive_id"], "ERROR", "Failed")

        return final if final is None else Data.NODES[device["hive_id"]]["Min_CT"]

    async def get_max_color_temp(self, device):
        """Get light maximum color temperature."""
        await self.log.log(device["hive_id"], "Extra", "Getting max colour temperature")
        online = await self.attr.online_offline(device["hive_id"])
        state = None
        final = None

        if device["hive_id"] in Data.products:
            if online:
                data = Data.products[device["hive_id"]]
                state = data["props"]["colourTemperature"]["min"]
                final = round((1 / state) * 1000000)
                Data.NODES[device["hive_id"]]["Max_CT"] = final
                await self.log.log(device["hive_id"], "Extra",
                                   "Max colour temp is {0}", info=final)
                if device["hive_id"] in Data. s_error_list:
                    Data. s_error_list.popitem(device["hive_id"])
            await self.log.error_check(device["hive_id"], "Extra", online)
        else:
            await self.log.error_check(device["hive_id"], "ERROR", "Failed")

        return final if final is None else Data.NODES[device["hive_id"]]["Max_CT"]

    async def get_color_temp(self, device):
        """Get light current color temperature."""
        await self.log.log(device["hive_id"], "Extra", "Getting colour temperature")
        online = await self.attr.online_offline(device["hive_id"])
        state = None
        final = None

        if device["hive_id"] in Data.products:
            if online:
                data = Data.products[device["hive_id"]]
                state = data["state"]["colourTemperature"]
                final = round((1 / state) * 1000000)
                Data.NODES[device["hive_id"]]["CT"] = final
                await self.log.log(device["hive_id"], "Extra", "Colour temp is {0}", info=final)
                if device["hive_id"] in Data. s_error_list:
                    Data. s_error_list.popitem(device["hive_id"])
            await self.log.error_check(device["hive_id"], "Extra", online)

        else:
            await self.log.error_check(device["hive_id"], "ERROR", "Failed")

        return final if final is None else Data.NODES[device["hive_id"]]["CT"]

    async def get_color(self, device):
        """Get light current colour"""
        await self.log.log(device["hive_id"], "Extra", "Getting colour info")
        online = await self.attr.online_offline(device["hive_id"])
        state = None
        final = None

        if device["hive_id"] in Data.products:
            if online:
                data = Data.products[device["hive_id"]]
                state = [
                    (data["state"]["hue"]) / 360,
                    (data["state"]["saturation"]) / 100,
                    (data["state"]["value"]) / 100,
                ]
                final = tuple(
                    int(i * 255)
                    for i in colorsys.hsv_to_rgb(state[0], state[1], state[2])
                )
                Data.NODES[device["hive_id"]]["Colour"] = final
                if device["hive_id"] in Data. s_error_list:
                    Data. s_error_list.popitem(device["hive_id"])
            await self.log.error_check(device["hive_id"], "Extra", online)
        else:
            await self.log.error_check(device["hive_id"], "ERROR", "Failed")

        return final if final is None else Data.NODES[device["hive_id"]]["Colour"]

    async def turn_off(self, device):
        """Set light to turn off."""
        await self.log.log(device["hive_id"], "Extra", "Turning off light")
        final = False

        if device["hive_id"] in Data.products:
            await self.session.hive_refresh_tokens()
            data = Data.products[device["hive_id"]]
            resp = await self.hive.set_state(data["type"],
                                             device["hive_id"], status="OFF")

            if resp["original"] == 200:
                final = True
                await self.session.get_devices(device["hive_id"])
                await self.log.log(device["hive_id"], "API", "Light off - " + device["hive_name"])
            else:
                await self.log.error_check(
                    device["hive_id"], "ERROR", "Failed_API", resp=resp["original"])

        return final

    async def turn_on(self, device, brightness, color_temp, color):
        """Set light to turn on."""
        await self.log.log(device["hive_id"], "Extra", "Turning on light")
        final = False

        if device["hive_id"] in Data.products:
            await self.session.hive_refresh_tokens()
            data = Data.products[device["hive_id"]]

            if brightness is not None:
                return await self.set_brightness(device, brightness)
            if color_temp is not None:
                return await self.set_color_temp(device, color_temp)
            if color is not None:
                return await self.set_color(device, color)

            resp = await self.hive.set_state(data["type"],
                                             device["hive_id"], status="ON")
            if resp["original"] == 200:
                final = True
                await self.session.get_devices(device["hive_id"])
                await self.log.log(device["hive_id"], "API", "Light on - " + device["hive_name"])
            else:
                await self.log.error_check(
                    device["hive_id"], "ERROR", "Failed_API", resp=resp["original"])

        return final

    async def set_brightness(self, device, n_brightness):
        """Set brightness of the light."""
        await self.log.log(device["hive_id"], "Extra", "Setting brightness")
        final = False

        if device["hive_id"] in Data.products:
            await self.session.hive_refresh_tokens()
            data = Data.products[device["hive_id"]]
            resp = await self.hive.set_state(data["type"],
                                             device["hive_id"], status="ON",
                                             brightness=n_brightness)
            if resp["original"] == 200:
                final = True
                await self.session.get_devices(device["hive_id"])
                await self.log.log(device["hive_id"], "API", "Brightness set to:  " + str(n_brightness))
            else:
                await self.log.error_check(
                    device["hive_id"], "ERROR", "Failed_API", resp=resp["original"])

        return final

    async def set_color_temp(self, device, color_temp):
        """Set light to turn on."""
        await self.log.log(device["hive_id"], "Extra", "Setting color temp")
        final = False

        if device["hive_id"] in Data.products:
            await self.session.hive_refresh_tokens()
            data = Data.products[device["hive_id"]]

            if data["type"] == "tuneablelight":
                resp = await self.hive.set_state(data["type"],
                                                 device["hive_id"],
                                                 colourTemperature=color_temp)
            else:
                await self.hive.set_state(data["type"], device["hive_id"],
                                          colourMode="COLOUR", hue="48", saturation="70",
                                          value="96")
                resp = await self.hive.set_state(data["type"],
                                                 device["hive_id"],
                                                 colourMode="WHITE",
                                                 colourTemperature=color_temp)

            if resp["original"] == 200:
                final = True
                await self.session.get_devices(device["hive_id"])
                await self.log.log(device["hive_id"], "API", "Colour temp set - " + device["hive_name"])
            else:
                await self.log.error_check(
                    device["hive_id"], "ERROR", "Failed_API", resp=resp["original"])

        return final

    async def set_color(self, device, new_color):
        """Set light to turn on."""
        await self.log.log(device["hive_id"], "Extra", "Setting color")
        final = False

        if device["hive_id"] in Data.products:
            await self.session.hive_refresh_tokens()
            data = Data.products[device["hive_id"]]

            resp = await self.hive.set_state(data["type"],
                                             device["hive_id"], colourMode="COLOUR",
                                             hue=str(new_color[0]), saturation=str(new_color[1]),
                                             value=str(new_color[2]))
            if resp["original"] == 200:
                final = True
                await self.session.get_devices(device["hive_id"])
                await self.log.log(device["hive_id"], "API", "Colour set - " + device["hive_name"])
            else:
                await self.log.error_check(
                    device["hive_id"], "ERROR", "Failed_API", resp=resp["original"])

        return final



""" Hive Session Module."""
import operator
import os
import time
import json
import asyncio
from typing import Optional
from aiohttp import ClientSession
from aiohttp.web import HTTPException
from datetime import datetime, timedelta


from .hive_data import Data
from .custom_logging import Logger
from .hive_async_api import Hive_Async
from .device_attributes import Attributes

TOKEN_EXPIRY = timedelta(minutes=60)


class Session:
    """Hive Session Code"""

    def __init__(self, websession: Optional[ClientSession] = None, init=None):
        """Initialise the base variable values."""
        self.update_lock = asyncio.Lock()
        self.api_lock = asyncio.Lock()
        self.hive = Hive_Async(websession)
        self.log = Logger()
        self.attr = Attributes()
        self.devices = {}
        self.type = "Session"
        self.init = init

    async def open_file(self, file):
        path = os.path.dirname(os.path.realpath(
            __file__)) + '/' + file
        with open(path, 'r') as j:
            data = json.loads(j.read())

        return data

    async def add_list(self, name, data, **kwargs):
        """Add entity to the list"""
        add = True
        if kwargs.get("custom") and not Data.s_sensors:
            add = False

        if add:
            formatted_data = {}
            try:
                formatted_data = {"hive_id": data.get("id", ""),
                                  "hive_name": data.get("state", {}).get("name", ""),
                                  "hive_type": data.get("type", ""),
                                  "ha_type": name
                                  }
                if kwargs.get("ha_name", "FALSE")[0] == " ":
                    kwargs["ha_name"] = data.get("state", {}).get(
                        "name", "") + kwargs["ha_name"]
                else:
                    formatted_data["ha_name"] = formatted_data["hive_name"]
                formatted_data.update(kwargs)
            except (KeyError):
                await self.log.log("No_ID", self.type, "Could not setup device - " + str(data))

            self.devices[name].append(formatted_data)
        return add

    async def update_interval(self, new_interval):
        "Update the scan interval."
        interval = timedelta(seconds=new_interval)
        if interval < timedelta(seconds=15):
            interval = timedelta(seconds=15)
        Data.s_interval_seconds = interval
        text = "Scan interval updated to " + str(Data.s_interval_seconds)
        await self.log.log("No_ID", self.type, text)

    async def use_file(self, username=None):
        "Update the scan interval."
        using_file = True if username == "use@file.com" else False
        if using_file:
            Data.s_file = True
            text = "Using a file."
            await self.log.log("No_ID", self.type, text)

    # DEPRECATED - no longer in use refesh token is the replacement.
    async def hive_api_logon(self):
        """Log in to the Hive API and get the Session Data."""

        if True in (Data.s_file, Data.s_token):
            return None
        else:
            await self.log.log("No_ID", self.type, "Checking if login is required.")

            expiry_time = Data.s_token_update + TOKEN_EXPIRY
            if datetime.now >= expiry_time or Data.s_tokens is None:
                await self.log.log("No_ID", self.type, "Attempting to login to Hive.")

                resp_p = await self.hive.login(Data.s_username, Data.s_password)

                if resp_p["original"] == 200:
                    info = resp_p["parsed"]
                    if "token" in info and "user" in info and "platform" in info:
                        Data.s_tokens = info["token"]
                        Data.s_token_update = datetime.now()

                        self.hive.urls.update(
                            {"base": info["platform"]["endpoint"]})

    async def hive_refresh_tokens(self):
        """Refresh Hive tokens."""
        updated = False

        if Data.s_file:
            await self.log.log("No_ID", self.type, "use_file is active - Cannot refresh tokens.")
            return None
        else:
            await self.log.log("No_ID", self.type, "Checking if new tokens are required")

            expiry_time = Data.s_token_update + TOKEN_EXPIRY
            if datetime.now() >= expiry_time or Data.s_entity_update_flag == "Y":
                await self.log.log("No_ID", self.type, "Attempting to refresh tokens.")
                resp_p = await self.hive.refresh_tokens()

                if resp_p["original"] == 200:
                    info = resp_p["parsed"]
                    if "token" in info:
                        Data.s_tokens.update({"token": info["token"]})
                        Data.s_tokens.update(
                            {"refreshToken": info["refreshToken"]})
                        Data.s_tokens.update(
                            {"accessToken": info["accessToken"]})
                        Data.s_token_update = datetime.now()
                        Data.s_entity_update_flag = "Ns"

                        self.hive.urls.update(
                            {"base": info["platform"]["endpoint"]})
                        updated = True
        return updated

    async def update_data(self, device):
        """Get latest data for Hive nodes - rate limiting."""
        await self.log.check_debuging(Data.d_list)
        await self.update_lock.acquire()
        updated = True
        try:
            ep = Data.s_last_update + Data.s_interval_seconds
            if datetime.now() >= ep:
                await self.get_devices(device["hive_id"])

            ep = Data.w_last_update + Data.w_interval_seconds
            if datetime.now() >= ep:
                await self.get_weather()
        except:
            updated = False
        finally:
            self.update_lock.release()

        return updated

    async def get_devices(self, n_id):
        """Get latest data for Hive nodes."""
        get_nodes_successful = False
        api_resp_d = None

        if self.api_lock.locked():
            return True

        try:
            await self.api_lock.acquire()
            await asyncio.sleep(1)
            if Data.s_file:
                await self.log.log(n_id, "API", "Using file")
                api_resp_d = await self.open_file("data.json")
            elif Data.s_tokens is not None:
                await self.log.log(n_id, "Session", "Getting hive device info")
                await self.hive_refresh_tokens()
                api_resp_d = await self.hive.get_all()
            else:
                await self.log.error_check(
                    n_id, "ERROR", "Failed_API", resp=api_resp_d["original"])
                asyncio.CancelledError('Could not get response from the API')

            api_resp_p = api_resp_d["parsed"]

            for hive_type in api_resp_p:
                if hive_type == "user":
                    Data.user = api_resp_p[hive_type]
                if hive_type == "products":
                    Data.products = {}
                    for a_product in api_resp_p[hive_type]:
                        Data.products.update({a_product["id"]: a_product})
                        Data.NODES.update({a_product["id"]: {"": ""}})
                if hive_type == "devices":
                    Data.devices = {}
                    for a_device in api_resp_p[hive_type]:
                        Data.devices.update({a_device["id"]: a_device})
                        Data.NODES.update({a_device["id"]: {"": ""}})
                if hive_type == "actions":
                    Data.actions = {}
                    for a_action in api_resp_p[hive_type]:
                        Data.actions.update({a_action["id"]: a_action})
                        Data.NODES.update({a_action["id"]: {"": ""}})

            Data.s_last_update = datetime.now()
            get_nodes_successful = True
        except (IOError, RuntimeError, ZeroDivisionError, ConnectionError):
            await self.log.log("No_ID", "Core_API", "Api didn't receive any data")
            get_nodes_successful = False
        finally:
            self.api_lock.release()

        return get_nodes_successful

    async def get_weather(self):
        """Get latest weather data from Hive."""
        get_weather_successful = False
        current_time = datetime.now()

        try:
            if Data.s_tokens is not None and not Data.s_file:
                await self.log.log("No_ID", self.type, "Getting Hive weather info.")
                await self.hive_refresh_tokens()
                weather_url = (
                    "?postcode=" +
                    Data.user.get("postcode", '') +
                    "&country=" + Data.user.get("countryCode", '')
                )

                resp = await self.hive.get_weather(weather_url)
                if resp["original"] == 200:
                    Data.w_icon = resp["parsed"]["weather"]["icon"]
                    Data.w_description = resp["parsed"]["weather"]["description"]
                    Data.w_temperature_unit = resp["parsed"]["weather"]["temperature"]["unit"]
                    Data.w_temperature_value = resp["parsed"]["weather"]["temperature"]["value"]
                    Data.w_nodeid = "HiveWeather"
                    Data.w_last_update = current_time
                    get_weather_successful = True

        except (IOError, RuntimeError, ZeroDivisionError):
            get_weather_successful = False

        return get_weather_successful

    @staticmethod
    async def p_minutes_to_time(minutes_to_convert):
        """Convert minutes string to datetime."""
        hours_converted, minutes_converted = divmod(minutes_to_convert, 60)
        converted_time = datetime.strptime(
            str(hours_converted) + ":" + str(minutes_converted), "%H:%M"
        )
        converted_time_string = converted_time.strftime("%H:%M")
        return converted_time_string

    @staticmethod
    async def p_get_schedule_nnl(self, hive_api_schedule):
        """Get the schedule now, next and later of a given nodes schedule."""
        schedule_now_and_next = {}
        date_time_now = datetime.now()
        date_time_now_day_int = date_time_now.today().weekday()

        days_t = (
            "monday",
            "tuesday",
            "wednesday",
            "thursday",
            "friday",
            "saturday",
            "sunday",
        )

        days_rolling_list = list(days_t[date_time_now_day_int:] + days_t)[:7]

        full_schedule_list = []

        for day_index in range(0, len(days_rolling_list)):
            current_day_schedule = hive_api_schedule[days_rolling_list[day_index]]
            current_day_schedule_sorted = sorted(
                current_day_schedule, key=operator.itemgetter("start"), reverse=False
            )

            for current_slot in range(0, len(current_day_schedule_sorted)):
                current_slot_custom = current_day_schedule_sorted[current_slot]

                slot_date = datetime.now() + timedelta(days=day_index)
                slot_time = await self.p_minutes_to_time(
                    current_slot_custom["start"])
                slot_time_date_s = slot_date.strftime(
                    "%d-%m-%Y") + " " + slot_time
                slot_time_date_dt = datetime.strptime(
                    slot_time_date_s, "%d-%m-%Y %H:%M"
                )
                if slot_time_date_dt <= date_time_now:
                    slot_time_date_dt = slot_time_date_dt + timedelta(days=7)

                current_slot_custom["Start_DateTime"] = slot_time_date_dt
                full_schedule_list.append(current_slot_custom)

        fsl_sorted = sorted(
            full_schedule_list, key=operator.itemgetter("Start_DateTime"), reverse=False
        )

        schedule_now = fsl_sorted[-1]
        schedule_next = fsl_sorted[0]
        schedule_later = fsl_sorted[1]

        schedule_now["Start_DateTime"] = schedule_now["Start_DateTime"] - timedelta(
            days=7
        )

        schedule_now["End_DateTime"] = schedule_next["Start_DateTime"]
        schedule_next["End_DateTime"] = schedule_later["Start_DateTime"]
        schedule_later["End_DateTime"] = fsl_sorted[2]["Start_DateTime"]

        schedule_now_and_next["now"] = schedule_now
        schedule_now_and_next["next"] = schedule_next
        schedule_now_and_next["later"] = schedule_later

        return schedule_now_and_next

    async def start_session(self, tokens, update, options):
        """Setup the Hive platform."""
        Data.s_sensors = options.get("add_sensors", True)
        Data.s_entity_update_flag = update
        await self.log.check_debuging(options.get("debug", []))
        await self.log.log("No_ID", self.type, "Initialising Hive Component.")
        await self.update_interval(options["scan_interval"])
        await self.use_file(options.get("username", False))

        if tokens is not None and not Data.s_file:
            await self.log.log("No_ID", self.type, "Logging into Hive.")
            Data.s_tokens.update(
                {"token": tokens["IdToken"]})
            Data.s_tokens.update(
                {"refreshToken": tokens["RefreshToken"]})
            Data.s_tokens.update(
                {"accessToken": tokens["AccessToken"]})
        elif Data.s_file:
            await self.log.log("No_ID", self.type, "Loading up a hive session with a preloaded file.")
        else:
            return "UNKNOWN_CONFIGURATION"

        try:
            self.hive.headers.update(
                {"authorization": Data.s_tokens.get("token")})
            await self.get_devices("No_ID")
            await self.get_weather()
        except HTTPException:
            return HTTPException

        if Data.devices == {} or Data.products == {}:
            await self.log.log("No_ID", self.type, "Failed to get data")
            return "INVALID_REAUTH"

        await self.log.log("No_ID", self.type, "Creating list of devices")
        self.devices["binary_sensor"] = []
        self.devices["climate"] = []
        self.devices["light"] = []
        self.devices["sensor"] = []
        self.devices["switch"] = []
        self.devices["water_heater"] = []

        for a_product in Data.products:
            p = Data.products[a_product]
            if Data.products[a_product]["type"] == "sense":
                await self.add_list("binary_sensor", p, ha_name="Glass Detection",
                                    hive_type="GLASS_BREAK", hub_id=p["parent"],
                                    device_id=p["parent"], device_name=p["state"]["name"])
                await self.add_list("binary_sensor", p, ha_name="Smoke Detection",
                                    hive_type="SMOKE_CO", hub_id=p["parent"],
                                    device_id=p["parent"], device_name=p["state"]["name"])
                await self.add_list("binary_sensor", p, ha_name="Dog Bark Detection",
                                    hive_type="DOG_BARK", hub_id=p["parent"],
                                    device_id=p["parent"], device_name=p["state"]["name"])

            if Data.products[a_product]["type"] in Data.HIVE_TYPES["Heating"]:
                device_id = p["props"].get("zone", p["id"])
                device_name = p["state"].get("name", "Heating")
                for device in Data.devices:
                    try:
                        if Data.devices[device]["type"] in Data.HIVE_TYPES["Thermo"]:
                            device = Data.devices[device]
                            if p["parent"] == device["props"]["zone"]:
                                device_id = device["id"]
                                device_name = device["state"]["name"]
                                break
                    except:
                        pass
                Data.MODE.append(p["id"])
                await self.add_list("climate", p, device_id=device_id, device_name=device_name,
                                    temperatureunit=Data.user["temperatureUnit"])
                await self.add_list("sensor", p, ha_name=" Current Temperature",
                                    hive_type="CurrentTemperature", device_id=device_id,
                                    device_name=device_name, custom=True)
                await self.add_list("sensor", p, ha_name=" Target Temperature",
                                    hive_type="TargetTemperature", device_id=device_id,
                                    device_name=device_name, custom=True)
                await self.add_list("sensor", p, ha_name=" State",
                                    hive_type="Heating_State", device_id=device_id,
                                    device_name=device_name, custom=True)
                await self.add_list("sensor", p, ha_name=" Mode",
                                    hive_type="Heating_Mode", device_id=device_id,
                                    device_name=device_name, custom=True)
                await self.add_list("sensor", p, ha_name=" Boost",
                                    hive_type="Heating_Boost", device_id=device_id,
                                    device_name=device_name, custom=True)
            if Data.products[a_product]["type"] in Data.HIVE_TYPES["Hotwater"]:
                device_id = p["props"].get("zone", p["id"])
                device_name = p["state"].get("name", "Hotwater")
                for device in Data.devices:
                    try:
                        if Data.devices[device]["type"] in Data.HIVE_TYPES["Thermo"]:
                            device = Data.devices[device]
                            if p["parent"] == device["props"]["zone"]:
                                device_id = device["id"]
                                device_name = device["state"]["name"]
                    except:
                        pass
                await self.add_list("water_heater", p, device_id=device_id, device_name=device_name)
                await self.add_list("sensor", p, ha_name=" State",
                                    hive_type="Hotwater_State", device_id=device_id,
                                    device_name=device_name, custom=True)
                await self.add_list("sensor", p, ha_name=" Mode",
                                    hive_type="Hotwater_Mode", device_id=device_id,
                                    device_name=device_name, custom=True)
                await self.add_list("sensor", p, ha_name=" Boost",
                                    hive_type="Hotwater_Boost", device_id=device_id,
                                    device_name=device_name, custom=True)
            if Data.products[a_product]["type"] in Data.HIVE_TYPES["Plug"]:
                Data.MODE.append(p["id"])
                await self.add_list("switch", p, device_id=p["id"], device_name=p["state"]["name"])
                await self.add_list("sensor", p, ha_name=" Mode", hive_type="Mode", device_id=p["id"],
                                    device_name=p["state"]["name"], custom=True)
                await self.add_list("sensor", p, ha_name=" Availability", hive_type="Availability", device_id=p["id"],
                                    device_name=p["state"]["name"], custom=True)
            if Data.products[a_product]["type"] in Data.HIVE_TYPES["Light"]:
                Data.MODE.append(p["id"])
                await self.add_list("light", p, device_id=p["id"], device_name=p["state"]["name"])
                await self.add_list("sensor", p, ha_name=" Mode", hive_type="Mode", device_id=p["id"],
                                    device_name=p["state"]["name"], custom=True)
                await self.add_list("sensor", p, ha_name=" Availability", hive_type="Availability", device_id=p["id"],
                                    device_name=p["state"]["name"], custom=True)
            if Data.products[a_product]["type"] in Data.HIVE_TYPES["Sensor"]:
                await self.add_list("binary_sensor", p, device_id=p["id"], device_name=p["state"]["name"])

        for a_device in Data.devices:
            d = Data.devices[a_device]
            if (
                    Data.devices[a_device]["type"] in Data.HIVE_TYPES["Thermo"]
                    or Data.devices[a_device]["type"] in Data.HIVE_TYPES["Sensor"]):
                Data.BATTERY.append(d["id"])
                await self.add_list("sensor", d, ha_name=" Battery Level",
                                    hive_type="Battery", device_id=d["id"], device_name=d["state"]["name"])
                await self.add_list("sensor", d, ha_name=" Availability",
                                    hive_type="Availability", device_id=d["id"], device_name=d["state"]["name"],
                                    custom=True)
            if Data.devices[a_device]["type"] in Data.HIVE_TYPES["Hub"]:
                await self.add_list("binary_sensor", d, ha_name=" Online Status",
                                    hive_type="Connectivity", device_id=d["id"], device_name=d["state"]["name"])

        for action in Data.actions:
            a = Data.actions[action]
            await self.add_list("switch", a, hive_name=a["name"],
                                ha_name=a["name"],
                                hive_type="action")

        if Data.w_nodeid == "HiveWeather":
            await self.add_list("sensor", {}, hive_id=Data.w_nodeid,
                                hive_name="Hive Weather",
                                ha_name="Hive Weather",
                                hive_type="Weather")

        await self.log.log("No_ID", self.type, "Hive component has initialised")

        return self.devices

    @staticmethod
    async def epochtime(date_time, pattern, action):
        """ date/time conversion to epoch"""
        if action == "to_epoch":
            pattern = "%d.%m.%Y %H:%M:%S"
            epochtime = int(time.mktime(
                time.strptime(str(date_time), pattern)))
            return epochtime
        elif action == "from_epoch":
            date = datetime.fromtimestamp(int(date_time)).strftime(pattern)
            return date



import os
import re

from setuptools import setup, find_packages


def requirements_from_file(filename='requirements.txt'):
    with open(os.path.join(os.path.dirname(__file__), filename)) as r:
        reqs = r.read().strip().split('\n')
    # Return non emtpy lines and non comments
    return [r for r in reqs if re.match(r"^\w+", r)]


setup(
    name="pyhiveapi",
    version="0.3_b14",
    description="A Python library to interface with the Hive API",
    long_description="A Python library to interface with the Hive API",
    url="https://github.com/Pyhive/pyhiveapi",
    package_data={
        'pyhiveapi.pyhiveapi': ['*.key', '*.json']
    },
    include_package_data=True,
    author="Rendili",
    author_email="rendili@outlook.com",
    license="MIT",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        #        'Topic :: Software Development :: API',
        "License :: OSI Approved :: MIT License",
        #        'Programming Language :: Python :: 3.4',
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
    python_requires=">=3.5.*",
    keywords="Hive API Library",
    packages=find_packages(exclude=["contrib", "docs", "tests"]),
    entry_points={"console_scripts": [
        "pyhiveapi=pyhiveapi.hive_session:Session"]},
    install_requires=requirements_from_file(),
    extras_require={
        'dev': requirements_from_file('requirements_test.txt')
    },
)



"""Hive Hub Module."""
import asyncio
from .custom_logging import Logger
from .device_attributes import Attributes
from .hive_data import Data


class Hub:
    """ Hive Hub Code. """

    def __init__(self):
        """Initialise."""
        self.log = Logger()
        self.attr = Attributes()
        self.type = "Hub"
        self.log_type = "Sensor"

    async def hub_smoke(self, device):
        """Get the online status of the Hive hub."""
        await self.log.log(device["hive_id"], "Extra", "Getting smoke detection status")
        online = await self.attr.online_offline(device["hub_id"])
        state = None
        final = None

        if device["hive_id"] in Data.products:
            if online:
                data = Data.products[device["hive_id"]]
                state = data["props"]["sensors"]["SMOKE_CO"]["active"]
                final = Data.HIVETOHA[self.type]["Smoke"].get(state, state)
                await self.log.log(device["hive_id"], "Extra", "Status is {0}", info=final)
                if device["hive_id"] in Data. s_error_list:
                    Data. s_error_list.popitem(device["hive_id"])
            await self.log.error_check(device["hive_id"], "Extra", final)
            final = Data.HIVETOHA[self.type]["Smoke"].get(state, state)
            Data.NODES[device["hive_id"]]["Smoke"] = final
        else:
            await self.log.error_check(device["hive_id"], "ERROR", "Failed")

        return final if final is None else Data.NODES[device["hive_id"]]["Smoke"]

    async def hub_dog_bark(self, device):
        """Get the online status of the Hive hub."""
        await self.log.log(device["hive_id"], "Extra", "Getting barking detection status")
        online = await self.attr.online_offline(device["hub_id"])
        state = None
        final = None

        if device["hive_id"] in Data.products:
            if online:
                data = Data.products[device["hive_id"]]
                state = data["props"]["sensors"]["DOG_BARK"]["active"]
                final = Data.HIVETOHA[self.type]["Dog"].get(state, state)
                await self.log.log(device["hive_id"], "Extra", "Status is {0}", info=final)
                if device["hive_id"] in Data. s_error_list:
                    Data. s_error_list.popitem(device["hive_id"])
            await self.log.error_check(device["hive_id"], "Extra", final)
            final = Data.HIVETOHA[self.type]["Dog"].get(state, state)
            Data.NODES[device["hive_id"]]["Dog"] = final
        else:
            await self.log.error_check(device["hive_id"], "ERROR", "Failed")

        return final if final is None else Data.NODES[device["hive_id"]]["Dog"]

    async def hub_glass(self, device):
        """Get the glass detected status from the Hive hub."""
        await self.log.log(device["hive_id"], "Extra", "Getting glass detection status")
        online = await self.attr.online_offline(device["hub_id"])
        state = None
        final = None

        if device["hive_id"] in Data.products:
            if online:
                data = Data.products[device["hive_id"]]
                state = data["props"]["sensors"]["GLASS_BREAK"]["active"]
                final = Data.HIVETOHA[self.type]["Glass"].get(state, state)
                await self.log.log(device["hive_id"], "Extra", "Status is {0}", info=final)
                if device["hive_id"] in Data. s_error_list:
                    Data. s_error_list.popitem(device["hive_id"])
            await self.log.error_check(device["hive_id"], "Extra", final)
            final = Data.HIVETOHA[self.type]["Glass"].get(state, state)
            Data.NODES[device["hive_id"]]["Glass"] = final
        else:
            await self.log.error_check(device["hive_id"], "ERROR", "Failed")

        return final if final is None else Data.NODES[device["hive_id"]]["Glass"]
